@@ -182,6 +182,7 @@ static struct {
     int           maskpri;
     int           logopt;
     int           delayopen;
+    int           openlog;
 } ctx = { 
     NULL,
     NULL,
@@ -190,6 +191,7 @@ static struct {
     NULL,
     LOG_UPTO(LOG_DEBUG),
     0,
+    FALSE,
     FALSE
 };
 
@@ -197,12 +199,16 @@ static struct {
 static void fsldebug(l2_level_t level, const char *message, ...)
 {
     va_list ap;
+    char cp[8];
 
     va_start(ap, message);
-    if (ctx.l2_fslnch != NULL)
-        l2_channel_vlog(ctx.l2_fslnch, level, message, ap);
-    else
-        vfprintf(stderr, message, ap);
+    /* note that a L2_ERR_USE is only returned by l2_channel_vlog() if the channel is not yet open */
+    if (ctx.l2_fslnch == NULL || (l2_channel_vlog(ctx.l2_fslnch, level, message, ap) == L2_ERR_USE))
+        if (level & L2_LEVEL_UPTO(L2_LEVEL_ERROR)) {
+            fprintf( stderr, "%s: ", l2_util_l2s(cp, sizeof(cp), '\0', level) == L2_OK ? cp[0]&=0xDF, cp : "Log");
+            vfprintf(stderr, message, ap);
+            fprintf( stderr, "\n");
+    }
     va_end(ap);
     return;
 }
@@ -256,32 +262,6 @@ CUS:
     return rc;
 }
 
-/* read a single file "fsl.xxx" into buffer */
-static fsl_rc_t readfile(buf_t *buffer, const char *ident)
-{
-    fsl_rc_t rc;
-    char *filename = NULL;
-
-    if (ident == NULL || buffer == NULL)
-        CU(FSL_ERR_ARG);
-    fsldebug(L2_LEVEL_TRACE, "readfile() ident=\"%s\")", ident);
-
-    if ((filename = (char *)malloc(strlen(FSL_CFGDIR) + 1 + 
-                                   strlen(FSL_PREFIX) + strlen(ident) + 1)) == NULL)
-        CU(FSL_ERR_MEM);
-    filename[0] = '\0';
-    strcat(filename, FSL_CFGDIR);
-    strcat(filename, "/");
-    strcat(filename, FSL_PREFIX);
-    strcat(filename, ident);
-
-    CU(appendfiletobuffer(buffer, filename));
-CUS:
-    if (filename != NULL)
-        free(filename);
-    return rc;
-}
-
 /* read all possible files "fsl.*" into buffer */
 static fsl_rc_t readallfiles(buf_t *buffer)
 {
@@ -324,25 +304,6 @@ CUS:
     return rc;
 }
 
-/* read a single or all possible files */
-static fsl_rc_t readfileorallfiles(buf_t *buffer, const char *ident)
-{
-    fsl_rc_t rv;
-
-    if (ident == NULL)
-        return FSL_ERR_ARG;
-    fsldebug(L2_LEVEL_TRACE, "readfileorallfiles() ident=\"%s\")", ident);
-
-    if ((rv = readfile(buffer, ident)) == FSL_OK)
-        return FSL_OK;
-    if (rv != FSL_ERR_SYS || errno != ENOENT)
-        return rv;
-    fsldebug(L2_LEVEL_TRACE, "readfileorallfiles: specific file does not exist, trying all files from directory");
-    if ((rv = readallfiles(buffer)) != FSL_OK)
-        return rv;
-    return FSL_OK;
-}
-
 /* OSSP cfg node tree traversal function (recursive) */
 static void traverse(cfg_t *cfg, cfg_node_t *cfgnode)
 {
@@ -661,6 +622,7 @@ void openlog(const char *ident, int logopt, int facility)
 
     /* properly handle repeated execution */
     closelog();
+    ctx.openlog = TRUE;
 
     /* create OSSP l2 environment for fsl itself (internal logging) */
     if ((argl2spec = getenv("FSL_DEBUG")) == NULL)
@@ -687,6 +649,7 @@ void openlog(const char *ident, int logopt, int facility)
     }
 
     /* tracing */
+    
     fsldebug(L2_LEVEL_TRACE, "openlog() ident=\"%s\", logopt=0x%.8lx, facility=0x%.8lx)", ident, logopt, facility);
 
     /* remember logopt and handle unsupported values */
@@ -745,8 +708,8 @@ void openlog(const char *ident, int logopt, int facility)
     strcat(cpISF, cpFacility);
 
     /* read configuration file(s) into buffer */
-    if ((rv = readfileorallfiles(&buf, ident)) != FSL_OK) {
-        fsldebug(L2_LEVEL_ERROR, "openlog: readfileorallfiles(buf, \"%s\") returned %d, system error: %s(%d)", ident, rv, strerror(errno), errno); CU(1); }
+    if ((rv = readallfiles(&buf)) != FSL_OK) {
+        fsldebug(L2_LEVEL_ERROR, "openlog: readallfiles() failed. Hint: last system error was \"%s\"(%d)", strerror(errno), errno); CU(1); }
 
     /* import configuration buffer into OSSP cfg node tree */
     if ((cfgrv = cfg_create(&cfg)) != CFG_OK) {
@@ -818,6 +781,7 @@ void closelog(void)
     ctx.maskpri = LOG_UPTO(LOG_DEBUG);
     ctx.logopt = 0;
     ctx.delayopen = FALSE;
+    ctx.openlog = FALSE;
     return;
 }
 
@@ -857,6 +821,10 @@ void vsyslog(int priority, const char *fmt, va_list args)
     l2_result_t l2rv;
     char *cp;
     
+    /* check for previous omitted attempt of initialization */
+    if (ctx.l2_nch == NULL && !ctx.openlog)
+        openlog("fsl", 0, LOG_SYSLOG);
+
     /* check for previous proper initialization */
     if (ctx.l2_nch == NULL)
         return;
